<?php

namespace Drupal\user_api_passwordless\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Flood\FloodInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user_api_passwordless\Event\PasswordlessLoginAfterUserLoadEvent;
use Drupal\user_api_passwordless\Event\PasswordlessLoginBeforeUserLoadEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Implement flood checks for passwordless login events.
 */
class PasswordlessLoginEventSubscriber implements EventSubscriberInterface {

  /**
   * Construct new EventSubscriber.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected FloodInterface $flood,
    protected RequestStack $requestStack,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      PasswordlessLoginBeforeUserLoadEvent::EVENT_NAME => [
        ['handleBeforeUserLoad'],
      ],
      PasswordlessLoginAfterUserLoadEvent::EVENT_NAME => [
        ['handleAfterUserLoad'],
      ],
    ];
  }

  /**
   * Check if IP has been blocked before user load.
   */
  public function handleBeforeUserLoad(PasswordlessLoginBeforeUserLoadEvent $event) {
    $floodConfig = $this->configFactory->get('user.flood');

    /** @var string[] $events */
    $names = Settings::get('user_api_passwordless.check_flood_ip_events', []);

    // Abort if any flood event is blocked.
    foreach ($names as $name) {
      if (!$this->flood->isAllowed($name, $floodConfig->get('ip_limit'), $floodConfig->get('ip_window'))) {
        $event->abort(
          'ip_blocked',
          new TranslatableMarkup('Too many failed login attempts for your IP address.'),
          Response::HTTP_FORBIDDEN
        );

        break;
      }
    }
  }

  /**
   * Check if user has been blocked after user load.
   */
  public function handleAfterUserLoad(PasswordlessLoginAfterUserLoadEvent $event) {
    $floodConfig = $this->configFactory->get('user.flood');
    $request = $this->requestStack->getCurrentRequest();

    // Get identifier depending on settings..
    if ($floodConfig->get('uid_only')) {
      $identifier = $event->user->id();
    }
    else {
      $identifier = $event->user->id() . '-' . $request->getClientIP();
    }

    /** @var string[] $events */
    $names = Settings::get('user_api_passwordless.check_flood_user_events', []);

    // Abort if any flood event is blocked.
    foreach ($names as $name) {
      if (!$this->flood->isAllowed($name, $floodConfig->get('user_limit'), $floodConfig->get('user_window'), $identifier)) {
        $event->abort(
          'user_blocked',
          new TranslatableMarkup('Too many failed login attempts for this account.'),
          Response::HTTP_FORBIDDEN
        );

        break;
      }
    }
  }

}
